<?php

namespace Drupal\one_time_password\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\Writer\PngWriter;

/**
 * A form for setting up one time passwords.
 */
class PasswordSetupForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'one_time_password_setup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, ?User $user = NULL) {
    $form_state->set('user', $user);
    if ($user->one_time_password->isEmpty()) {
      $form['instructions'] = [
        '#theme' => 'one_time_password_enable_instructions',
      ];
      $form['generate'] = [
        '#type' => 'submit',
        '#value' => $this->t('Enable Two-Factor Authentication'),
        '#submit' => [[$this, 'enableTwoFactorAuth']],
      ];
    }
    else {
      $password = $user->one_time_password->getOneTimePassword();
      // @todo Remove BC support in
      // https://www.drupal.org/project/one_time_password/issues/3465857
      if (method_exists(QrCode::class, 'writeDataUri')) {
        $qr_code = new QrCode($password->getProvisioningUri());
        $qr_code->setSize(200);
        // @phpstan-ignore-next-line
        $data_uri = $qr_code->writeDataUri();
      }
      else {
        $qr_code = QrCode::create($password->getProvisioningUri())
          ->setSize(200);
        $writer = new PngWriter();
        $result = $writer->write($qr_code);
        $data_uri = $result->getDataUri();
      }

      $form['qr_code'] = [
        '#theme' => 'image',
        '#uri' => $data_uri,
      ];
      $form['instructions'] = [
        '#theme' => 'one_time_password_setup_instructions',
      ];
      $form['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Disable Two-Factor Authentication'),
        '#submit' => [[$this, 'disableTwoFactorAuth']],
        '#button_type' => 'danger',
      ];
    }

    $form['#cache']['max-age'] = 0;
    return $form;
  }

  /**
   * Submit callback to enable two-factor authentication.
   */
  public function enableTwoFactorAuth(array &$form, FormStateInterface $form_state) {
    $user = $form_state->get('user');
    $user->one_time_password->regenerateOneTimePassword();
    $user->save();
    $this->messenger()->addMessage($this->t('Two-factor authentication has been enabled. See the instructions below for setting up your one time password.'));
  }

  /**
   * Submit callback to disable two-factor authentication.
   */
  public function disableTwoFactorAuth(array &$form, FormStateInterface $form_state) {
    $user = $form_state->get('user');
    $user->one_time_password = [];
    $user->save();
    $this->messenger()->addMessage($this->t('Two-factor authentication has been disabled for this account.'));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

}
